#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>

/* Vrtices de digrafos so representados por objetos do tipo Vertex. */
#define Vertex int


struct grafo {
   int V;
   int A;
   int **adj;
};
typedef struct grafo *Grafo;

struct vertice {
   int pos;
   int qual;
   int id;
   int sub;
   int posicao_matriz;
   int s1;
   int s2;
};
typedef struct vertice *Vertice;

/* Esta funo aloca uma matriz com linhas 0..r-1 e colunas 0..c-1. Cada elemento da matriz recebe valor val. */
int **criaMatriz( int r, int c, int val) {
   Vertex i, j;
   int **m = malloc( r * sizeof (int *));
   for (i = 0; i < r; i++)
      m[i] = malloc( c * sizeof (int));
   for (i = 0; i < r; i++)
      for (j = 0; j < c; j++)
         m[i][j] = val;
   return m;
}

char **criaMatriz2( int r, int c, char val) {
   Vertex i, j;
   char **m = malloc( r * sizeof (char *));
   for (i = 0; i < r; i++)
      m[i] = malloc( c * sizeof (char));
   for (i = 0; i < r; i++)
      for (j = 0; j < c; j++)
         m[i][j] = val;
   return m;
}

/* A seguinte funo devolve um novo grafo com vrtices 0 1 .. V-1 e nenhum arco. */
Grafo criaGrafo( int V) {
   Grafo G = malloc( sizeof *G);
   G->V = V;
   G->A = 0;
   G->adj = criaMatriz( V, V, -1);
   return G;
}

Vertice criaVert( int qtd) {
   int i;
   Vertice *v = malloc(qtd * sizeof (int *));
   for (i=0; i<qtd; i++)
        v[i] = malloc( sizeof(int));
   return v;
}

/*  Esta funo insere um arco v-w no digrafo G.
    A funo supe que v e w so distintos, positivos e menores que G->V.
    Se o digrafo j tem arco v-w, a funo no faz nada. */
void insereAresta( Grafo G, Vertex v, Vertex w, int i) {
   if (G->adj[v][w] == -1) {
      G->adj[v][w] = i;
      G->A++;
   }
}

/* Esta funo remove do digrafo G o arco que tem ponta inicial v e ponta final w.
A funo supe que v e w so distintos, positivos e menores que G->V.
Se no existe arco v-w, a funo no faz nada. */
void removeAresta( Grafo G, Vertex v, Vertex w) {
   if (G->adj[v][w] == 1) {
      G->adj[v][w] = -1;
      G->A--;
   }
}
/* Para cada vrtice v do digrafo G, esta funo imprime, em uma linha, todos os vrtices adjacentes a v. */
void imprimeGrafo( Grafo G) {
   Vertex v, w;
   for (v = 0; v < G->V; v++) {
      printf( "%2d:", v);
      for (w = 0; w < G->V; w++)
         if (G->adj[v][w] > 0)
            printf( " %2d", w);
      printf( "\n");
   }
}

void imprimeMatriz( Grafo G) {
   Vertex v, w;
   int r=0, c=0;

   printf("\n\n ");
   for (c = 0; c < G->V; c++)
        printf("  %d", c);
   for (v = 0; v < G->V; v++) {
      printf("\n%d", r);
      for (w = 0; w < G->V; w++)
            printf( " %2d", G->adj[v][w]);
      r++;
   }
}

int Conta_Fragmentos(); //conta o numero de fragmentos a agrupar
int Conta_Contigs(); //conta o numero de vertices
int getInfo(int, int);
int ini_seq(Grafo G);
Vertice prox_seq(Grafo G);
void junction(Grafo G, Vertice *v, int n);
char* Le_Fragmento(char n[4]);

int main()
{
    int frag, contigs, v, i, s, p, q;
    Vertex a=1, b=2;
    Grafo G;
    Vertice *id, prox;

    contigs = Conta_Contigs();
    printf("\n\n contigs: %d", contigs);
    frag = Conta_Fragmentos();

    G = criaGrafo(frag);
    id = criaVert(frag);
    for (i=0; i<contigs; i++){
        a = getInfo(i, 1); //primeira sequencia
        b = getInfo(i, 2); //segunda sequencia
        s = getInfo(i, 3); //sequencia com casamento no sufixo
        p = getInfo(i, 4); //posicao do casamento
        q = getInfo(i, 5); //qualidade
        id[i]->pos = p;
        id[i]->qual = q;
        id[i]->id = i;
        id[i]->sub = s;
        id[i]->s1 = a;
        id[i]->s2 = b;

        if(s==a)
            insereAresta(G, b, a, p);//vertice b comea dentro do vertice a
        if(s==b)
            insereAresta(G, a, b, p);//vertice a comea dentro do vertice b
    }

    for(i=0; i<contigs; i++){
        printf("\nid: %d\n", id[i]->id);
        printf("qual: %d\n", id[i]->qual);
        printf("pos: %d\n", id[i]->pos);
        printf("sub: %d\n", id[i]->sub);
        printf("s1: %d\n", id[i]->s1);
        printf("s2: %d\n", id[i]->s2);
        printf("\n\n");
    }

    imprimeMatriz(G);
    printf("\n\n");

    prox = prox_seq(G);

    printf("\n\n\n");

    junction(G, id, frag);
    printf("\n\n\n");

    imprimeGrafo(G);
    return 0;
}

void junction(Grafo G, Vertice *v, int n){
    int cadeia, i=0, j=0, k=0, aux,  r = 0, w=0, y=0, l=0, cont=0, ver=-1;
    char c[211], **m, id1[4] = {'/0'}, msc[411];
    FILE *S;
    Vertice p, p_ant;

    S = fopen("saida.fasta", "w+");

    if (S == NULL) {
        printf("\nArquivo saida.txt nao pode ser aberto.");
        printf("\nUse alguma tecla para finalizar !");
        getch();
    }

    cadeia = ini_seq(G);
    itoa(v[cadeia]->sub, id1, 10);//converte a string em um numero inteiro e armazena na variavel id1
    v[cadeia]->posicao_matriz = 0;
    strcpy(c, Le_Fragmento(id1));

    while(c[i]!='\0'){//escreve o primeiro fragmento no arquivo saida.txt
        fputc(c[i], S);
        i++;
    }
    cont=i;
    strcpy(msc, c);
    fputc('\n', S);


    for(i=0; i<n; i++){//calcula em qual posio deve iniciar o proximo fragmento e o escreve no arquivo
        p = prox_seq(G);
        if(p->id != ver){
            ver = p->id;
            //printf("\n\nid: %d\n\n\n", p->id);
            if(p->id >= 0){
                itoa(p->id, id1, 10);
                strcpy(c, Le_Fragmento(id1));
                //p->pos
                //printf("\n\npos: %d\n\n\n", p->pos);
                j=0;
                while(j < p->pos){//enquanto nao chegar na posio correta para iniciar, preenche com "-"
                    fputc('-', S);
                    j++;
                }
                w=0;
                while(c[w]!='\0'){//enquanto nao acabarem os caracteres do fragmento, escreve-os no arquivo
                    fputc(c[w], S);
                    l = p->pos+w;
                    if(l>cont)
                        cont++;
                    msc[l] = c[w];
                    w++;
                }
                fputc('\n', S);
            }
        }else{
            i=n;
        }
    }
    fputc('\n', S);
    i=0;
    while(i<=cont){
        fputc(msc[i], S);
        i++;
    }
    fclose(S);
}
/* Esta funo le um determinado fragmento do arquivo, passado por parametro, e o retorna como vetor*/
char* Le_Fragmento(char n[4]){
    FILE *P;
    int i=0;
    char* s = (char *) malloc (211);
    char aux[4] = {'/0'}, ini;

    P = fopen("fragmentos.fasta", "r");
    if (P == NULL) {
        printf("\nArquivo fragmentos.fasta nao pode ser aberto.");
        printf("\nUse alguma tecla para finalizar !");
        getch();
    }

    while (!feof(P))
    {
        i=0;
        ini = fgetc(P); //le o primeiro caracter da linha
        if(ini=='>'){ //verifica se  uma linha de comentrio do formato fasta
            ini = fgetc(P); //le o S

            ini = fgetc(P); //le o numero da sequencia

            while(ini!='\n'){
                aux[i] = ini; //recebe o numero da sequencia
                i++;
                ini = fgetc(P);//le o proximo digito
            }
        }

        i = 0;
        if(strcmp(aux,n)==0){//verifica se o id do fragmento passado por parmetro  igual ao fragmento a ser copiado
            ini = fgetc(P);//le o primeiro caracter
            s[i] = ini;//armazena-o em s
            while(ini=='A'||ini=='G'||ini=='T'||ini=='C'){//considerar apenas se for uma base
                ini = fgetc(P);//le o proximo
                i++;
                s[i] = ini;//armazena em s
            }
            s[i] = '\0';
            fclose(P);
            return(s);//retorna o vetor contendo o fragmento lido
        }
    }


}


Vertice prox_seq(Grafo G){
    Vertice p;
    int i, j, k, aux;

    p->id = -1;

    for(i=0; i<G->V; i++){
        for(j=0; j<G->V; j++){
            if(G->adj[i][j]>=0){
                for(k=0; k<G->V; k++){
                    aux = G->adj[j][k];
                    if(aux == -2){

                        p->id = i;
                        p->pos = G->adj[i][j];
                       //
                        p->sub = j;
                        G->adj[i][0] = -2; //marca ele como ja inserido
                       // printf("\nentrou\ni= %d j= %d k=%d, pos=%d", i, j, k, G->adj[i][0]);
                        j = G->V + 1;
                        i = G->V + 1;
                        k = G->V + 1;
                    }

                }
            }
        }
    }
    return p;
}

/*
Esta funo busca pelo primeiro vertice a ser inserido na matriz.
Este vrtice no pode ter seu prefixo envolvido com nenhum dos vertices restantes
*/
int ini_seq(Grafo G){
    Vertice p = criaVert(1);
    int i, j, m, n, aux=0, ver=0;

    p->id = -1;
    p->qual = -1;

    for(i=0; i<G->V; i++){//verifica se este vrtice possui casamento em seu prefixo
        for(j=0; j<G->V; j++){
            if(G->adj[i][j]<0)//casso nao haja nenhum casamento em seu prefixo incrementamos a variavel
                aux++;
        }

        if(aux==j){// se nao ouve casamento com nenhum outro vertice
            if(p->id<0){//se ainda nao foi escolhido nenhum vertice para retornar
                p->id = i;//escolhe
                G->adj[i][0] = -2; //marca ele como ja inserido
            }
            else{
                for(n=0; n<G->V; n++){//se ja houver um vrtice para retorno, precisamos ver o que possui mais casamentos em seu sufixo
                    for(m=0; m<G->V; m++){
                        if(G->adj[m][n]>=0){//
                            ver++;  //incremento do numero de casamentos em seu sufixo
                        }
                    }
                    if(p->qual<ver){//se o valor antigo a ser retornado tiver menos casamentos em seu sufixo, trocamos
                        p->id = i;
                        p->qual = ver;
                        G->adj[i][0] = -2; //marca ele como ja inserido
                    }
                }
            }
        }
    }
    m = p->id;
    return m;
}

int Conta_Fragmentos(){
    FILE *P;
    char *result, Linha[211];
    int n=0, ini;
    P = fopen("fragmentos.fasta", "r");

    if (P == NULL) {
        printf("\nArquivo fragmentos.fasta nao pode ser aberto.");
        printf("\nUse alguma tecla para finalizar !");
        getch();
    }

    while (!feof(P))
    {
        ini = fgetc(P); //le o primeiro caracter da linha
        if(ini=='>'){ //verifica se  uma linha de comentrio do formato fasta
            result = fgets(Linha, 211, P);  //ignora a linha de comentario, at o '\n'
            ini = fgetc(P); //le o primeiro caracter da proxima linha
            n++;
        }
    }
    fclose(P);
    return(n);
}

int Conta_Contigs(){
    FILE *P;
    char *result, Linha[211];
    int n=0, ini;
    P = fopen("contigs.fasta", "r");

    if (P == NULL) {
        printf("\nArquivo contigs.fasta nao pode ser aberto.");
        printf("\nUse alguma tecla para finalizar !");
        getch();
    }

    while (!feof(P))
    {
        ini = fgetc(P); //le o primeiro caracter da linha
        if(ini=='>'){ //verifica se  uma linha de comentrio do formato fasta
            n++;
            result = fgets(Linha, 211, P);  //ignora a linha de comentario, l at o '\n'
        }
    }
    fclose(P);
    return(n);
}

int getInfo(int i, int info){
    FILE *P;
    char Linha[211], ini, x[6]={'\0'}, *result;
    int n=0, res;
    P = fopen("contigs.fasta", "r");

    if (P == NULL) {
        printf("\nArquivo contigs.fasta nao pode ser aberto.");
        printf("\nUse alguma tecla para finalizar !");
        getch();
    }

    for (n=0; n<i; n++){
        result = fgets(Linha, 211, P);
    }

    switch(info){
        case 1:
            ini = fgetc(P); //le o primeiro caracter da linha
            while(ini!='a'){ //enquanto nao encontrar a primeira sequencia
                ini = fgetc(P); //le at encontrar quem  a primeira sequencia
                //printf("\nini: %c", ini);
            }
            i=0;
            ini = fgetc(P); //procura pelo numero da sequencia
            while(ini!=','){//enquanto nao acabar de ler o numero
                x[i] = ini;//armazena o numero da sequencia no vertor x
                ini = fgetc(P); //le o proximo caracter
                i++;
            }
            res = atoi(x);//transforma o numero que esta no vetor em um inteiro
        break;

        case 2:
            ini = fgetc(P); //le o primeiro caracter da linha
            while(ini!='b'){ //enquanto nao encontrar a segunda sequencia
                ini = fgetc(P); //le at encontrar quem  a segunda sequencia
            }
            i=0;
            ini = fgetc(P); //le o primeiro digito do numero da sequncia
            while(ini!=')'){//enquanto nao acabarem os digitos
                x[i] = ini;//armazena em x
                ini = fgetc(P); //le o proximo
                i++;
            }
            res = atoi(x);//transforma o numero que esta no vetor em um inteiro
        break;

        case 3:
            ini = fgetc(P); //leo primeiro caracter
            while(ini!='S'){ //enquanto nao encontrar a orientao da sequencia
                ini = fgetc(P); //le at encontrar qual  a orientacao
            }
            i=0;
            ini = fgetc(P); //le o primeiro digito do sentido do casamento
            while(ini!=' '){//enquanto nao acabarem os digitos
                x[i] = ini;//armazena em x
                ini = fgetc(P); //le o proximo
                i++;
            }
            res = atoi(x);//transforma o numero que esta no vetor em um inteiro
        break;
        case 4:
            ini = fgetc(P); //le o primeiro caracter
            while(ini!='P'){ //enquanto nao encontrar a posicao do casamento
                ini = fgetc(P); //le at encontrar a posicao do casamento
            }
            i=0;
            ini = fgetc(P); //le o primeiro digito do numero referente a posicao do casamento
            while(ini!=' '){//enquanto nao acabarem os digitos
                x[i] = ini;//armazena em x
                ini = fgetc(P); //le o proximo
                i++;
            }
            res = atoi(x);//transforma o numero que esta no vetor em um inteiro
        break;
        case 5:
            ini = fgetc(P); //le o primeiro caracter
            while(ini!='Q'){ //enquanto nao encontrar a qualidade do casamento
                ini = fgetc(P); //le at encontrar a qualidade
            }
            i=0;
            ini = fgetc(P); //le o primeiro digito do numero referente a qualidade do casamento
            while(ini!='\n'){//enquanto nao acabarem os digitos
                x[i] = ini;//armazena em x
                ini = fgetc(P); //le o proximo
                i++;
            }
            res = atoi(x);//transforma o numero que esta no vetor em um inteiro
        break;
    }

    fclose(P);
    return(res);
}
