#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>

/* Vrtices de digrafos so representados por objetos do tipo Vertex. */
#define Vertex int


struct grafo {
   int V;
   int A;
   int **adj;
};
typedef struct grafo *Grafo;

struct vertice {
   int pos;
   int qual;
   int idContig;
};
typedef struct vertice *Vertice;

/* Esta funo aloca uma matriz com linhas 0..r-1 e colunas 0..c-1. Cada elemento da matriz recebe valor val. */
int **criaMatriz( int r, int c, int val) {
   Vertex i, j;
   int **m = malloc( r * sizeof (int *));
   for (i = 0; i < r; i++)
      m[i] = malloc( c * sizeof (int));
   for (i = 0; i < r; i++)
      for (j = 0; j < c; j++)
         m[i][j] = val;
   return m;
}

/* A seguinte funo devolve um novo grafo com vrtices 0 1 .. V-1 e nenhum arco. */
Grafo criaGrafo( int V) {
   Grafo G = malloc( sizeof *G);
   G->V = V;
   G->A = 0;
   G->adj = criaMatriz( V, V, -1);
   return G;
}

Vertice criaVert( int qtd) {
   int i;
   Vertice *v = malloc(qtd * sizeof (int *));
   for (i=0; i<qtd; i++)
        v[i] = malloc( sizeof(int));
   return v;
}
void imprimeInfo( Vertice v) {
   printf("\nposicao: $d", v->pos);
   printf("\nqualidade: $d", v->qual);
   printf("\nid: $d", v->idContig);
}

/* Esta funo insere um arco v-w no digrafo G. A funo supe que v e w so distintos, positivos e menores que G->V.
Se o digrafo j tem arco v-w, a funo no faz nada. */
void insereAresta( Grafo G, Vertex v, Vertex w, int i) {
   if (G->adj[v][w] == -1) {
      G->adj[v][w] = i;
      G->A++;
   }
}

/* Esta funo remove do digrafo G o arco que tem ponta inicial v e ponta final w.
A funo supe que v e w so distintos, positivos e menores que G->V.
Se no existe arco v-w, a funo no faz nada. */
void removeAresta( Grafo G, Vertex v, Vertex w) {
   if (G->adj[v][w] == 1) {
      G->adj[v][w] = -1;
      G->A--;
   }
}
/* Para cada vrtice v do digrafo G, esta funo imprime, em uma linha, todos os vrtices adjacentes a v. */
void imprimeGrafo( Grafo G) {
   Vertex v, w;
   for (v = 0; v < G->V; v++) {
      printf( "%2d:", v);
      for (w = 0; w < G->V; w++)
         if (G->adj[v][w] > 0)
            printf( " %2d", w);
      printf( "\n");
   }
}

void imprimeMatriz( Grafo G) {
   Vertex v, w;
   int r=0, c=0;

   printf("\n\n ");
   for (c = 0; c < G->V; c++)
        printf("  %d", c);
   for (v = 0; v < G->V; v++) {
      printf("\n%d", r);
      for (w = 0; w < G->V; w++)
            printf( " %2d", G->adj[v][w]);
      r++;
   }
}

int Conta_Fragmentos(); //conta o numero de fragmentos a agrupar
int Conta_Contigs(); //conta o numero de vertices
int getInfo(int, int);

int main()
{
    int frag, contigs, i, s, p, q;
    Vertex a=1, b=2;
    Grafo G;
    Vertice *id;

    contigs = Conta_Contigs();
    printf("\n\n contigs: %d", contigs);
    frag = Conta_Fragmentos();

    G = criaGrafo(frag);
    id = criaVert(frag);
    for (i=0; i<contigs; i++){
        a = getInfo(i, 1);
        b = getInfo(i, 2);
        s = getInfo(i, 3);
        p = getInfo(i, 4);
        q = getInfo(i, 5);
        id[i]->pos = p;
        id[i]->qual = q;
        id[i]->idContig = i;

        if(s==a)
            insereAresta(G, b, a, q);//vertice b comea dentro do vertice a
        if(s==b)
            insereAresta(G, a, b, q);//vertice a comea dentro do vertice b
    }

    imprimeMatriz(G);

    printf("\n\n");
    imprimeGrafo(G);
    return 0;
}

int Conta_Fragmentos(){
    FILE *P;
    char *result, Linha[211];
    int n=0, ini;
    P = fopen("fragmentos.fasta", "r");

    if (P == NULL) {
        printf("\nArquivo fragmentos.fasta nao pode ser aberto.");
        printf("\nUse alguma tecla para finalizar !");
        getch();
    }

    while (!feof(P))
    {
        ini = fgetc(P); //le o primeiro caracter da linha
        if(ini=='>'){ //verifica se  uma linha de comentrio do formato fasta
            result = fgets(Linha, 211, P);  //ignora a linha de comentario, l at 99 caracteres ou at o '\n'
            ini = fgetc(P); //le o primeiro caracter da proxima linha
            n++;
        }
    }
    fclose(P);
    return(n);
}

int Conta_Contigs(){
    FILE *P;
    char *result, Linha[211];
    int n=0, ini;
    P = fopen("contigs.fasta", "r");

    if (P == NULL) {
        printf("\nArquivo contigs.fasta nao pode ser aberto.");
        printf("\nUse alguma tecla para finalizar !");
        getch();
    }

    while (!feof(P))
    {
        ini = fgetc(P); //le o primeiro caracter da linha
        if(ini=='>'){ //verifica se  uma linha de comentrio do formato fasta
            n++;
            result = fgets(Linha, 211, P);  //ignora a linha de comentario, l at 99 caracteres ou at o '\n'
        }
    }
    fclose(P);
    return(n);
}

int getInfo(int i, int info){
    FILE *P;
    char Linha[211], ini, x[6]={'\0'}, *result;
    int n=0, res;
    P = fopen("contigs.fasta", "r");

    if (P == NULL) {
        printf("\nArquivo contigs.fasta nao pode ser aberto.");
        printf("\nUse alguma tecla para finalizar !");
        getch();
    }

    for (n=0; n<i; n++){
        result = fgets(Linha, 211, P);  //ignora a linha de comentario, l at 99 caracteres ou at o '\n'
    }

    switch(info){
        case 1:
            ini = fgetc(P); //le o primeiro caracter da linha
            while(ini!='a'){ //verifica se  uma linha de comentrio do formato fasta
                ini = fgetc(P); //le o primeiro caracter da proxima linha
                //printf("\nini: %c", ini);
            }
            i=0;
            ini = fgetc(P); //le o primeiro caracter da proxima linha
            while(ini!=','){
                x[i] = ini;
                ini = fgetc(P); //le o primeiro caracter da proxima linha
                i++;
            }
            res = atoi(x);
        break;

        case 2:
            ini = fgetc(P); //le o primeiro caracter da linha
            while(ini!='b'){ //verifica se  uma linha de comentrio do formato fasta
                ini = fgetc(P); //le o primeiro caracter da proxima linha
            }
            i=0;
            ini = fgetc(P); //le o primeiro caracter da proxima linha
            while(ini!=')'){
                x[i] = ini;
                ini = fgetc(P); //le o primeiro caracter da proxima linha
                i++;
            }
            res = atoi(x);
        break;

        case 3:
            ini = fgetc(P); //le o primeiro caracter da linha
            while(ini!='S'){ //verifica se  uma linha de comentrio do formato fasta
                ini = fgetc(P); //le o primeiro caracter da proxima linha
            }
            i=0;
            ini = fgetc(P); //le o primeiro caracter da proxima linha
            while(ini!=' '){
                x[i] = ini;
                ini = fgetc(P); //le o primeiro caracter da proxima linha
                i++;
            }
            res = atoi(x);
        break;
        case 4:
            ini = fgetc(P); //le o primeiro caracter da linha
            while(ini!='P'){ //verifica se  uma linha de comentrio do formato fasta
                ini = fgetc(P); //le o primeiro caracter da proxima linha
            }
            i=0;
            ini = fgetc(P); //le o primeiro caracter da proxima linha
            while(ini!=' '){
                x[i] = ini;
                ini = fgetc(P); //le o primeiro caracter da proxima linha
                i++;
            }
            res = atoi(x);
        break;
        case 5:
            ini = fgetc(P); //le o primeiro caracter da linha
            while(ini!='Q'){ //verifica se  uma linha de comentrio do formato fasta
                ini = fgetc(P); //le o primeiro caracter da proxima linha
            }
            i=0;
            ini = fgetc(P); //le o primeiro caracter da proxima linha
            while(ini!='\n'){
                x[i] = ini;
                ini = fgetc(P); //le o primeiro caracter da proxima linha
                i++;
            }
            res = atoi(x);
        break;
    }

    fclose(P);
    return(res);
}
